package net.blumia.pcmdroid

import android.net.Uri
import android.util.Log
import androidx.lifecycle.*
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.async
import kotlinx.coroutines.flow.first
import kotlinx.coroutines.flow.firstOrNull
import kotlinx.coroutines.launch
import kotlinx.coroutines.runBlocking
import net.blumia.pcmdroid.model.Folder
import net.blumia.pcmdroid.model.Server
import net.blumia.pcmdroid.model.Song
import net.blumia.pcmdroid.model.parseFromJsonString
import net.blumia.pcmdroid.repository.ServerRepository
import net.blumia.pcmdroid.viewmodel.AddServerViewModel
import okhttp3.FormBody
import okhttp3.HttpUrl.Companion.toHttpUrl
import okhttp3.OkHttpClient
import okhttp3.Request

class MainViewModel(private val repository: ServerRepository) : ViewModel() {

    private val httpClient: OkHttpClient = OkHttpClient()

    val servers: LiveData<List<Server>> = repository.allServers.asLiveData()
    fun addServer(server: Server) = viewModelScope.launch {
        repository.insert(server)
    }

    private val _currentServer: MutableLiveData<Server> = MutableLiveData(null)
    val currentServer: LiveData<Server> = _currentServer;
    fun setCurrentServer(srv: Server) {
        _currentServer.postValue(srv)
    }

    private val _currentFolder: MutableLiveData<Folder> = MutableLiveData(null)
    val currentFolder: LiveData<Folder> = _currentFolder
    fun setCurrentFolder(folder: Folder) {
        _currentFolder.postValue(folder)
    }

    private val _drawerFolders: MutableLiveData<List<Folder>> = MutableLiveData(listOf())
    val drawerFolders: LiveData<List<Folder>> = _drawerFolders

    private val _folders: MutableLiveData<List<Folder>> = MutableLiveData(listOf())
    val folders: LiveData<List<Folder>> = _folders

    private val _songs: MutableLiveData<List<Song>> = MutableLiveData(listOf())
    val songs: LiveData<List<Song>> = _songs

    fun fetchServer(srv: Server) {
        setCurrentServer(srv)
        viewModelScope.launch(context = Dispatchers.IO) {

            val formBody = FormBody.Builder()
                .add("do", "getfilelist")
                .build()

            val request = Request.Builder()
//                .url("http://localhost/")
                .url(srv.url)
                .post(formBody)
                .build()

            var mediaRoot = srv.mediaBaseUrl
            if (mediaRoot.startsWith("//")) mediaRoot = request.url.scheme + mediaRoot

            try {
                httpClient.newCall(request).execute().use { response ->
                    if (!response.isSuccessful) {
                        // TODO: non-200 response will go to here too.
                        return@use
                    }
                    val pair = parseFromJsonString(mediaRoot, "", response.body!!.string())
                    _drawerFolders.postValue(pair.first!!)
//                    setCurrentServer(srv)
                    Log.d("vvv", drawerFolders.value.toString())
                }
            } catch (e: java.io.IOException) {
                e.printStackTrace()
            }

//            Thread.sleep(5_000)
        }
    }

    fun fetchFolder(folder: Folder) {
        if (currentServer.value == null) return
        val srv = currentServer.value!!

        viewModelScope.launch(context = Dispatchers.IO) {

            val formBody = FormBody.Builder()
                .add("do", "getfilelist")
                .add("folder", folder.folderPath)
                .build()

            val request = Request.Builder()
                .url(srv.url)
                .post(formBody)
                .build()

            httpClient.newCall(request).execute().use { response ->
                if (!response.isSuccessful) {
                    // TODO: non-200 response will go to here too.
                    return@use
                }
                val pair = parseFromJsonString(srv.mediaBaseUrl, folder.folderPath, response.body!!.string())
                _folders.postValue(pair.first)
                _songs.postValue(pair.second)

                setCurrentFolder(folder)

                Log.d("vvv", folders.value.toString())
            }
        }
    }

    fun getServerById(id: Long): Server {
        return runBlocking {
            repository.serverById(id).first()
        }
    }

    fun editServer(srv: Server) {
        runBlocking {
            repository.update(srv)
        }
    }

    fun deleteServer(srv: Server) {
        runBlocking {
            repository.delete(srv)
        }
    }
}

class MainViewModelFactory(private val repository: ServerRepository) : ViewModelProvider.Factory {
    override fun <T : ViewModel> create(modelClass: Class<T>): T {
        if (modelClass.isAssignableFrom(MainViewModel::class.java)) {
            @Suppress("UNCHECKED_CAST")
            return MainViewModel(repository) as T
        } else if (modelClass.isAssignableFrom(AddServerViewModel::class.java)) {
            @Suppress("UNCHECKED_CAST")
            return AddServerViewModel() as T
        }
        throw IllegalArgumentException("Unknown ViewModel class")
    }
}