package net.blumia.pcmdroid.ui.screen.main

import android.net.Uri
import androidx.compose.foundation.*
import androidx.compose.foundation.gestures.scrollable
import androidx.compose.foundation.layout.*
import androidx.compose.material.*
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.*
import androidx.compose.material.icons.rounded.FeaturedPlayList
import androidx.compose.material.icons.rounded.PlayArrow
import androidx.compose.material.icons.rounded.PlayCircle
import androidx.compose.material.icons.rounded.PlayCircleOutline
import androidx.compose.runtime.*
import androidx.compose.runtime.getValue
import androidx.compose.runtime.setValue
import androidx.compose.runtime.livedata.observeAsState
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.vector.ImageVector
import androidx.compose.ui.layout.VerticalAlignmentLine
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import kotlinx.coroutines.launch
import net.blumia.pcmdroid.R
import net.blumia.pcmdroid.model.Folder
import net.blumia.pcmdroid.model.Server
import net.blumia.pcmdroid.model.Song

@Preview(showBackground = true)
@OptIn(ExperimentalMaterialApi::class)
@Composable
fun FileList(
    modifier: Modifier = Modifier,
    currentFolder: Folder? = null,
    folders: List<Folder> = listOf(),
    songs: List<Song> = listOf(),
    onFolderItemClicked: (Folder) -> Unit = {},
    onSongItemClicked: (Song, List<Song>) -> Unit = { _, _ -> },
) {
    Column(
        modifier = modifier
    ) {

        Breadcrumb(
            folder = currentFolder,
            onFolderClicked = onFolderItemClicked,
        )

        folders.forEach { folder ->
            ListItem(
                modifier = Modifier.clickable {
                    onFolderItemClicked(folder)
                },
                icon = {
                    Icon(
                        imageVector = Icons.Filled.FolderOpen,
                        contentDescription = null,
                    )
                },
                text = {
                    Text(folder.displayName())
                }
            )
        }

        songs.forEach { song ->
            ListItem(
                modifier = Modifier.clickable {
                    onSongItemClicked(song, songs)
                },
                icon = {
                    Icon(
                        imageVector = Icons.Filled.MusicNote,
                        contentDescription = null
                    )
                },
                text = {
                    Text(song.displayName())
                }
            )
        }
    }
}

@Preview(showBackground = true)
@OptIn(ExperimentalMaterialApi::class)
@Composable
fun NowPlaying() {
    Column(
        modifier = Modifier
            .fillMaxWidth(),
    ) {
        LinearProgressIndicator(
            modifier = Modifier.fillMaxWidth(),
            progress = 0f,
        )

        Row(
            modifier = Modifier
                .fillMaxWidth()
                .height(56.dp),
            verticalAlignment = Alignment.CenterVertically,
        ) {
            Image(
                painter = painterResource(id = R.drawable.ic_launcher_background),
                contentDescription = "Album cover",
            )

            ListItem(
                text = {
                    Text(
                        text = "Not playing at all.",
                    )
                },
                trailing = {
                    IconButton(onClick = { /*TODO*/ }) {
                        Icon(imageVector = Icons.Rounded.PlayCircleOutline, contentDescription = "Menu")
                    }
                }
            )
        }
    }
}

@OptIn(ExperimentalMaterialApi::class)
@Composable
fun MainPlayer(
    currentServer: Server? = null,
    servers: List<Server> = listOf(),
    drawerFolders: List<Folder> = listOf(),
    currentFolder: Folder? = null,
    folders: List<Folder> = listOf(),
    songs: List<Song> = listOf(),
    addServerActionTriggered: () -> Unit = {},
    onServerItemIconClicked: (Server) -> Unit = {},
    onFolderItemClicked: (Folder) -> Unit = {},
    onSongItemClicked: (Song, List<Song>) -> Unit = { _, _ -> },
    onEditServerActionTriggered: (Server) -> Unit = {},
    onDeleteServerActionTriggered: (Server) -> Unit = {},
    settingsActionTriggered: () -> Unit = {},
) {
    val bottomDrawerState = rememberBottomDrawerState(initialValue = BottomDrawerValue.Closed)
    val bottomDrawerScope = rememberCoroutineScope()
    BottomDrawer(
        drawerState = bottomDrawerState,
        gesturesEnabled = bottomDrawerState.isOpen,
        drawerContent = {
            if (currentServer != null) {
                ServerSummaryDrawer(
                    server = currentServer,
                    onEditActionTriggered = {
                        onEditServerActionTriggered(currentServer)
                        bottomDrawerScope.launch {
                            bottomDrawerState.close()
                        }
                    },
                    onDeleteActionTriggered = {
                        onDeleteServerActionTriggered(currentServer)
                        bottomDrawerScope.launch {
                            bottomDrawerState.close()
                        }
                    },
                )
            } else {
                Text(
                    "TODO: play control and playlist?",
                    modifier = Modifier.fillMaxWidth().padding(16.dp),
                )
            }
        },
    ) {
        val scaffoldState = rememberScaffoldState()
        val scaffoldScope = rememberCoroutineScope()

        Scaffold(
            scaffoldState = scaffoldState,
            topBar = {
                TopAppBar(
                    title = { Text(stringResource(id = R.string.app_name)) },
                    navigationIcon = {
                        IconButton(onClick = {
                            scaffoldScope.launch {
                                scaffoldState.drawerState.open()
                            }
                        }) {
                            Icon(imageVector = Icons.Filled.Menu, contentDescription = "Open drawer")
                        }
                    },
                    actions = {
                        IconButton(onClick = { /*TODO*/ }) {
                            Icon(imageVector = Icons.Filled.MoreVert, contentDescription = "Menu")
                        }
                    }
                )
            },
            drawerContent = {
                SidebarContent(
                    currentServer = currentServer,
                    servers = servers,
                    drawerFolders = drawerFolders,
                    addServerActionTriggered = addServerActionTriggered,
                    onServerItemIconClicked = onServerItemIconClicked,
                    onServerMenuIconClicked = {
                        bottomDrawerScope.launch {
                            bottomDrawerState.open()
                        }
                    },
                    onDrawerFolderItemClicked = onFolderItemClicked,
                    settingsActionTriggered = settingsActionTriggered,
                )
            }
        ) {
            Column(
                modifier = Modifier.fillMaxSize()
            ) {
                FileList(
                    modifier = Modifier
                        .weight(1f, fill = true)
                        .verticalScroll(rememberScrollState()),
                    currentFolder, folders, songs,
                    onFolderItemClicked, onSongItemClicked
                )
                NowPlaying()
            }
        }
    }
}

@Preview(showBackground = true)
@Composable
fun MainPlayerPreview() {
    MainPlayer(
        currentServer = null,
        servers = listOf(),
        drawerFolders  = listOf(),
        currentFolder = null,
        folders = listOf(),
        songs = listOf(),
    )
}